@@ -109,6 +109,7 @@ public:
     void setNavigationStyleSheetFromFile(const QUrl& stylesheetfile);
     void setNavigationBehavior(int behavior);
     void setShowQuickButtons(bool show);
+    void setTransparency(bool useTransparency);
 
 signals:
 
@@ -150,7 +151,7 @@ private slots:
     void newWindow();
     void showManageProfilesDialog();
     void activateMenuBar();
-    void showSettingsDialog();
+    void showSettingsDialog(const bool showProfilePage = false);
     void showShortcutsDialog();
     void newFromProfile(Profile::Ptr profile);
     void activeViewChanged(SessionController* controller);
@@ -210,6 +211,7 @@ private:
 
     bool _menuBarInitialVisibility;
     bool _menuBarInitialVisibilityApplied;
+    bool _useTransparency;
 };
 }
 
